{{
***************************************
*  Graphics Template                  *
*  Author: Albert Emanuel Milani      *
*  Copyright (c) 2014 AEM             *
*  See end of file for terms of use.  *
***************************************
}}

CON

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
  _stack = (x_tiles*y_tiles*128 + 100) >> 2   'accomodate display memory and stack

  x_tiles = 16
  y_tiles = 12

  paramcount = 21       
  bitmap_base = $8000-(x_tiles*y_tiles*64)
  display_base = $8000-(x_tiles*y_tiles*128)
  

VAR

  long  vga_status      'status: off/visible/invisible  read-only       (21 contiguous longs)
  long  vga_enable      'enable: off/on                 write-only
  long  vga_pins        'pins: byte(2),topbit(3)        write-only
  long  vga_mode        'mode: interlace,hpol,vpol      write-only
  long  vga_videobase   'video base @word               write-only
  long  vga_colorbase   'color base @long               write-only              
  long  vga_hc          'horizontal cells               write-only
  long  vga_vc          'vertical cells                 write-only
  long  vga_hx          'horizontal cell expansion      write-only
  long  vga_vx          'vertical cell expansion        write-only
  long  vga_ho          'horizontal offset              write-only
  long  vga_vo          'vertical offset                write-only
  long  vga_hd          'horizontal display pixels      write-only
  long  vga_hf          'horizontal front-porch pixels  write-only
  long  vga_hs          'horizontal sync pixels         write-only
  long  vga_hb          'horizontal back-porch pixels   write-only
  long  vga_vd          'vertical display lines         write-only
  long  vga_vf          'vertical front-porch lines     write-only
  long  vga_vs          'vertical sync lines            write-only
  long  vga_vb          'vertical back-porch lines      write-only
  long  vga_rate        'pixel rate (Hz)                write-only

  word  screen[x_tiles * y_tiles]
  long  colors[64]
  

OBJ

  vga   : "vga"
  gr    : "graphics"
  

PUB start | i,dx,dy

  'start vga
  longmove(@vga_status,@vgaparams,paramcount)
  vga_videobase:=@screen
  vga_colorbase:=@colors
  vga.start(@vga_status)

  'init colors
  repeat i from 0 to 63
    colors[i]:=%%3330_2220_1110_0000

  'init tile screen
  repeat dx from 0 to vga_hc-1
    repeat dy from 0 to vga_vc-1
      screen[dy*vga_hc+dx]:=display_base>>6+dy+dx*vga_vc+(0<<10)

  'start and setup graphics
  gr.start
  gr.setup(x_tiles,y_tiles,x_tiles*8,y_tiles*8,bitmap_base)
  gr.colorwidth(3,0)
  gr.textmode(3,3,6,%0101)

  repeat

    'clear bitmap
    gr.clear

    'insert user code here

    'copy bitmap to display
    gr.copy(display_base)

PUB sin(angle) : dy

'' Get sine of angle (0-8191)

  dy := angle << 1 & $FFE
  if angle & $800
    dy := word[$F000 - dy]
  else
    dy := word[$E000 + dy]
  if angle & $1000
    -dy

PUB cos(angle) : dx

'' Get cosine of angle (0-8191)

  dx := sin(angle + $800)

DAT

vgaparams               long    0               'status
                        long    1               'enable
                        long    %010_111        'pins
                        long    %0011           'mode
                        long    0               'videobase
                        long    0               'colorbase
                        long    x_tiles         'hc
                        long    y_tiles         'vc
                        long    1               'hx
                        long    1               'vx
                        long    0               'ho
                        long    0               'vo
                        long    512             'hd
                        long    16              'hf
                        long    96              'hs
                        long    48              'hb
                        long    380             'vd
                        long    11              'vf
                        long    2               'vs
                        long    31              'vb
                        long    20_000_000      'rate

{{

┌──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┐
│                                                   TERMS OF USE: MIT License                                                  │                                                            
├──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┤
│Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    │ 
│files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    │
│modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software│
│is furnished to do so, subject to the following conditions:                                                                   │
│                                                                                                                              │
│The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.│
│                                                                                                                              │
│THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          │
│WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         │
│COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   │
│ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         │
└──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┘
}}
